//
//  ViewController.swift
//  NativePMExampleApp
//
//  Created by Vilas on 25/03/21.
//  Copyright © 2021 CocoaPods. All rights reserved.
//

import UIKit
import ConsentViewController

class ViewController: UIViewController {
    var activityIndicator = UIActivityIndicatorView(style: .white)

    @IBAction func onLoadPMTap(_ sender: Any) {
        consentManager.loadGDPRPrivacyManager(withId: "529562")
    }

    lazy var consentManager: SPConsentManager = { SPConsentManager(
        accountId: 22,
        propertyName: try! SPPropertyName("appletv.demo"),
        campaigns: SPCampaigns(
            gdpr: SPCampaign()
        ),
        delegate: self
    )}()

    override func viewDidLoad() {
        super.viewDidLoad()
        consentManager.loadMessage()
    }
}

extension ViewController: SPDelegate {
    func onSPUIReady(_ controller: SPMessageViewController) {
        print("onSPUIReady")
        present(controller, animated: true)
    }

    func onAction(_ action: SPAction, from controller: SPMessageViewController) {
        print("onAction:", action)
    }

    func onSPUIFinished(_ controller: SPMessageViewController) {
        print("onSPUIFinished")
        dismiss(animated: true)
    }

    func onConsentReady(userData: SPUserData) {
        print("onConsentReady:", userData)
    }

    func onError(error: SPError) {
        stopActivityIndicator()
        print("ERROR: ", error.description )
    }
}

// MARK: - UI Utils
extension ViewController {
    func showActivityIndicator(on parentView: UIView?) {
        if let containerView = parentView {
            activityIndicator.startAnimating()
            activityIndicator.translatesAutoresizingMaskIntoConstraints = false
            activityIndicator.color = .black
            containerView.addSubview(activityIndicator)
            NSLayoutConstraint.activate([
                activityIndicator.centerXAnchor.constraint(equalTo: containerView.centerXAnchor),
                activityIndicator.centerYAnchor.constraint(equalTo: containerView.centerYAnchor)
            ])
        }
    }

    func stopActivityIndicator() {
        activityIndicator.stopAnimating()
        activityIndicator.removeFromSuperview()
    }
}

